@@ -10,16 +10,14 @@
 // License for the specific language governing permissions and limitations under
 // the License.
 
-var sandbox = null;
-
-function init_sandbox() {
+function create_sandbox() {
   try {
     // if possible, use evalcx (not always available)
-    sandbox = evalcx('');
+    var sandbox = evalcx('');
     sandbox.emit = Views.emit;
     sandbox.sum = Views.sum;
     sandbox.log = log;
-    sandbox.toJSON = Couch.toJSON;
+    sandbox.toJSON = JSON.stringify;
     sandbox.JSON = JSON;
     sandbox.provides = Mime.provides;
     sandbox.registerType = Mime.registerType;
@@ -28,10 +26,16 @@ function init_sandbox() {
     sandbox.getRow = Render.getRow;
     sandbox.isArray = isArray;
   } catch (e) {
-    log(e.toSource());
+    var sandbox = {};
   }
+  return sandbox;
+};
+
+function create_filter_sandbox() {
+  var sandbox = create_sandbox();
+  sandbox.emit = Filter.emit;
+  return sandbox;
 };
-init_sandbox();
 
 // Commands are in the form of json arrays:
 // ["commandname",..optional args...]\n
@@ -43,8 +47,10 @@ var DDoc = (function() {
     "lists"     : Render.list,
     "shows"    : Render.show,
     "filters"   : Filter.filter,
+    "views"     : Filter.filter_view, 
     "updates"  : Render.update,
-    "validate_doc_update" : Validate.validate
+    "validate_doc_update" : Validate.validate,
+    "rewrites"  : Render.rewrite
   };
   var ddocs = {};
   return {
@@ -73,14 +79,19 @@ var DDoc = (function() {
           var point = ddoc;
           for (var i=0; i < funPath.length; i++) {
             if (i+1 == funPath.length) {
-              fun = point[funPath[i]]
+              var fun = point[funPath[i]];
+              if (!fun) {
+                throw(["error","not_found",
+                       "missing " + funPath[0] + " function " + funPath[i] +
+                       " on design doc " + ddocId]);
+              }
               if (typeof fun != "function") {
-                fun = Couch.compileFunction(fun, ddoc);
+                fun = Couch.compileFunction(fun, ddoc, funPath.join('.'));
                 // cache the compiled fun on the ddoc
-                point[funPath[i]] = fun
+                point[funPath[i]] = fun;
               };
             } else {
-              point = point[funPath[i]]              
+              point = point[funPath[i]];
             }
           };
 
@@ -101,6 +112,7 @@ var Loop = function() {
     // "view"    : Views.handler,
     "reset"    : State.reset,
     "add_fun"  : State.addFun,
+    "add_lib"  : State.addLib,
     "map_doc"  : Views.mapDoc,
     "reduce"   : Views.reduce,
     "rereduce" : Views.rereduce
@@ -116,12 +128,14 @@ var Loop = function() {
     } else if (e.error && e.reason) {
       // compatibility with old error format
       respond(["error", e.error, e.reason]);
+    } else if (e.name) {
+      respond(["error", e.name, e]);
     } else {
-      respond(["error","unnamed_error",e.toSource()]);
+      respond(["error","unnamed_error",e.toSource ? e.toSource() : e.stack]);
     }
   };
   while (line = readline()) {
-    cmd = eval('('+line+')');
+    cmd = JSON.parse(line);
     State.line_length = line.length;
     try {
       cmdkey = cmd.shift();
@@ -138,4 +152,14 @@ var Loop = function() {
   };
 };
 
+// Seal all the globals to prevent modification.
+seal(Couch, true);
+seal(JSON, true);
+seal(Mime, true);
+seal(Render, true);
+seal(Filter, true);
+seal(Views, true);
+seal(isArray, true);
+seal(log, true);
+
 Loop();
