@@ -21,7 +21,7 @@ var resolveModule = function(names, mod, root) {
       parent : mod.parent,
       id : mod.id,
       exports : {}
-    }
+    };
   }
   // we need to traverse the path
   var n = names.shift();
@@ -31,22 +31,22 @@ var resolveModule = function(names, mod, root) {
     }
     return resolveModule(names, {
       id : mod.id.slice(0, mod.id.lastIndexOf('/')),
-      parent : mod.parent.parent.parent,
-      current : mod.parent.parent.current
+      parent : mod.parent.parent,
+      current : mod.parent.current
     });
   } else if (n == '.') {
     if (!mod.parent) {
       throw ["error", "invalid_require_path", 'Object has no parent '+JSON.stringify(mod.current)];
     }
     return resolveModule(names, {
-      parent : mod.parent.parent,
-      current : mod.parent.current,
+      parent : mod.parent,
+      current : mod.current,
       id : mod.id
     });
   } else if (root) {
     mod = {current : root};
   }
-  if (!mod.current[n]) {
+  if (mod.current[n] === undefined) {
     throw ["error", "invalid_require_path", 'Object has no property "'+n+'". '+JSON.stringify(mod.current)];
   }
   return resolveModule(names, {
@@ -58,40 +58,62 @@ var resolveModule = function(names, mod, root) {
 
 var Couch = {
   // moving this away from global so we can move to json2.js later
-  toJSON : function (val) {
-    return JSON.stringify(val);
-  },
-  compileFunction : function(source, ddoc) {    
+  compileFunction : function(source, ddoc, name) {
     if (!source) throw(["error","not_found","missing function"]);
-    try {
-      if (sandbox) {
-        if (ddoc) {
-          var require = function(name, module) {
-            module = module || {};
-            var newModule = resolveModule(name.split('/'), module, ddoc);
-            var s = "function (module, exports, require) { " + newModule.current + " }";
-            try {
-              var func = sandbox ? evalcx(s, sandbox) : eval(s);
-              func.apply(sandbox, [newModule, newModule.exports, function(name) {return require(name, newModule)}]);
-            } catch(e) { 
-              throw ["error","compilation_error","Module require('"+name+"') raised error "+e.toSource()]; 
-            }
-            return newModule.exports;
-          }
-          sandbox.require = require;
+
+    var functionObject = null;
+    var sandbox = create_sandbox();
+
+    var require = function(name, module) {
+      module = module || {};
+      var newModule = resolveModule(name.split('/'), module.parent, ddoc);
+      if (!ddoc._module_cache.hasOwnProperty(newModule.id)) {
+        // create empty exports object before executing the module,
+        // stops circular requires from filling the stack
+        ddoc._module_cache[newModule.id] = {};
+        var s = "function (module, exports, require) { " + newModule.current + "\n }";
+        try {
+          var func = sandbox ? evalcx(s, sandbox, newModule.id) : eval(s);
+          func.apply(sandbox, [newModule, newModule.exports, function(name) {
+            return require(name, newModule);
+          }]);
+        } catch(e) {
+          throw [
+            "error",
+            "compilation_error",
+            "Module require('" +name+ "') raised error " +
+            (e.toSource ? e.toSource() : e.stack)
+          ];
         }
-        var functionObject = evalcx(source, sandbox);
+        ddoc._module_cache[newModule.id] = newModule.exports;
+      }
+      return ddoc._module_cache[newModule.id];
+    };
+
+    if (ddoc) {
+      sandbox.require = require;
+      if (!ddoc._module_cache) ddoc._module_cache = {};
+    }
+
+    try {
+      if(typeof CoffeeScript === "undefined") {
+        functionObject = evalcx(source, sandbox, name);
       } else {
-        var functionObject = eval(source);
+        var transpiled = CoffeeScript.compile(source, {bare: true});
+        functionObject = evalcx(transpiled, sandbox, name);
       }
     } catch (err) {
-      throw(["error", "compilation_error", err.toSource() + " (" + source + ")"]);
+      throw([
+        "error",
+        "compilation_error",
+        (err.toSource ? err.toSource() : err.stack) + " (" + source + ")"
+      ]);
     };
     if (typeof(functionObject) == "function") {
       return functionObject;
     } else {
       throw(["error","compilation_error",
-        "Expression does not eval to a function. (" + source.toSource() + ")"]);
+        "Expression does not eval to a function. (" + source.toString() + ")"]);
     };
   },
   recursivelySeal : function(obj) {
@@ -108,15 +130,15 @@ var Couch = {
       }
     }
   }
-}
+};
 
-// prints the object as JSON, and rescues and logs any toJSON() related errors
+// prints the object as JSON, and rescues and logs any JSON.stringify() related errors
 function respond(obj) {
   try {
-    print(Couch.toJSON(obj));
+    print(JSON.stringify(obj));
   } catch(e) {
     log("Error converting object to JSON: " + e.toString());
-    log("error on obj: "+ obj.toSource());
+    log("error on obj: "+ (obj.toSource ? obj.toSource() : obj.toString()));
   }
 };
 
@@ -125,7 +147,7 @@ function log(message) {
   if (typeof message == "xml") {
     message = message.toXMLString();
   } else if (typeof message != "string") {
-    message = Couch.toJSON(message);
+    message = JSON.stringify(message);
   }
   respond(["log", String(message)]);
 };
